// factor_rsa.c
// version: 2.0
// author : Chad Rempp
// date   : July 21st 2006
//
// This program uses a unique method to factor large semiprime RSA numbers. It
// chooses two random numbers, multiplies them together and checks if the
// product is the RSA number. If the product is the RSA number obviously the
// two chosen numbers make up the prime factorization of the RSA number.

// Version 2.0
// * Changed the method from finding two random products to finding a random
//   divisor, checking if the remainder is 0.
// * Set the upper bound of the random divisor to the square root of the RSA
//   number.
// * Added file saving when factors found. Not functioning, fprintf is broke.
// * Added timing

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <gmp.h>

// Which platform?
#define MS_DOS
//#define UNIX

// Threaded?
//#define THREADS

// MPI? (for beowulf clusters or other multi computing enviroments, unix only)
//#define MPI

// Uncomment this to output debug messages.
//#define DEBUG

// This is the RSA number to be used.
// For testing use 55108321 = 7919 * 6959 and set SIZE_B to 13
#define RSA_NUM "74037563479561712828046796097429573142593188889231289084936232638972765034028266276891996419625117843995894330502127585370118968098286733173273108930900552505116877063299072396380786710086096962537934650563796359"
// (41072957 * 41972101)
//#define RSA_NUM "1723918299572657"
//#define RSA_NUM "55108321"

#define FILE_NAME "rsa_output.txt"

main (void){
  int counter, ch;
  unsigned long seed, i, j;
  gmp_randstate_t s;
  mpz_t factor1, factor2;
  mpz_t rsa, up_bound, div, rem;
  FILE *out;

  clock_t start, end;

  //----------INITIALIZATION----------------------------------------------------
  counter = 0;
  seed = time(NULL); // system time

  mpz_init(factor1);
  mpz_init(factor2);
  mpz_init(up_bound);
  mpz_init(div);
  mpz_init(rem);
  mpz_init_set_str(rsa, RSA_NUM, 10);

  gmp_randinit_default(s);
  gmp_randseed_ui(s, seed);
  
  mpz_sqrt(up_bound, rsa);

  //----------MAIN LOOP---------------------------------------------------------
  gmp_printf("Trying to factor RSA Number :\n%Zd\n", rsa);
  
  start = clock();
  
  #ifdef MS_DOS
  	printf("Press the q to quit\n");
	while (!kbhit()){
  #endif
  #ifdef UNIX
  	printf("CTL-C to quit\n");
  	while (1){
  #endif
	#ifdef DEBUG
      gmp_printf("RSA Number       :\n%Zd\n", rsa);
      gmp_printf("Upper bound : %Zd\n", up_bound);
	#endif
	
	// Get divisor
	mpz_urandomm(div, s, up_bound);
	// Increase divisor by two to avoid 0 and 1 (division by zero case and
	// trivial case)
	mpz_add_ui(div, div, 2);
	#ifdef DEBUG
	  gmp_printf("Divisor is   : %Zd\n", div);
	#endif
	
	// Get remainder
	mpz_mod(rem, rsa, div);
	#ifdef DEBUG
	  gmp_printf("Remainder is : %Zd\n", rem);
	#endif
	
	// Test if the product is the RSA number.
    if(mpz_cmp_ui(rem, 0) == 0){
	  //int wrote;
	  printf("\nTHE FACTORS HAVE BEEN FOUND!!!\n");
	  
	  mpz_set(factor1, div);
	  mpz_cdiv_q(factor2, rsa, div);
	  
	  gmp_printf("FACTOR 1 : %Zd\n", factor1);
      gmp_printf("FACTOR 2 : %Zd\n", factor2);
	  
	  //out = fopen(FILE_NAME, "w");
      //if (out == NULL){
	  //  perror("Unable to open file");
	  //  exit(EXIT_FAILURE);
      //}	  
      //printf("%d\n", fprintf(out, "Factors:\n"));
      //mpz_out_str(out, 10, rop[0]);
      //fprintf(out, "\n");
      //mpz_out_str(out, 10, rop[1]);
      //fclose(out);
      goto done;
    }
    #ifndef DEBUG
      if (counter == 0){
		  printf("\r-");
	  }
	  else if (counter == 10000){
		  printf("\r\\");
	  }
	  else if (counter == 20000){
		  printf("\r|");
	  }
	  else if (counter == 30000){
		  printf("\r/");
		  counter = 0;
	  }
	  counter++;
	#endif
  }
  done:
  end = clock();
  printf("\n");

  printf("Running time : %f seconds\n",
  		(double)(end - start)/(double)CLOCKS_PER_SEC);

  while ((ch = getchar()) != '\n' && ch != EOF);
  
  // Clean up.
  gmp_randclear(s);
  mpz_clear(rsa);
  mpz_clear(div);
  mpz_clear(rem);
  mpz_clear(up_bound);
  mpz_clear(factor1);
  mpz_clear(factor2);

  return 0;
}


